//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインバス (X68030)
//

#pragma once

#include "mainbus.h"

class BankRAMDevice;
class ExtRAMDevice;
class X68kIODevice;

class X68kMainbus final : public MainbusDevice
{
	using inherited = MainbusDevice;
 public:
	X68kMainbus();
	~X68kMainbus() override;

	bool Init() override;
	bool InitMainbus() override;
	void ResetByMPU() override;
	void ResetHard(bool poweron) override;

	uint64 Peek8(uint32 addr) override;
	uint64 Poke8(uint32 addr, uint32 data) override;

	uint64 Read8(fcaddr fcaddr) override;
	uint64 Read16(fcaddr fcaddr) override;
	uint64 Read32(fcaddr fcaddr) override;
	uint64 Write8(fcaddr fcaddr, uint32 data) override;
	uint64 Write16(fcaddr fcaddr, uint32 data) override;
	uint64 Write32(fcaddr fcaddr, uint32 data) override;

 private:
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);
	DECLARE_MONITOR_CALLBACK(MonitorUpdateAccStat);

	inline bool IsIntio(uint32) const;

	// ブートページを切り替える。
	void SwitchBootPage(bool isrom) override;

	std::unique_ptr<ExtRAMDevice> pExtRAM /*{}*/;
	std::unique_ptr<X68kIODevice> pX68kIO /*{}*/;
	// どこに置く?
	std::unique_ptr<IODevice> pInterrupt {};

	std::array<bool, 256> is_intio {};
	// 拡張メモリの開始位置 [MB]
	int extstart {};
	// 拡張メモリのサイズ [MB]
	int extsize {};

	// アクセス状況はとりあえず 1GB 分を 1048 * 1MB ずつ。
	std::array<uint8, 1024> accstat_read {};
	std::array<uint8, 1024> accstat_write {};

	Monitor monitor { this };
	Monitor monitor_accstat { this };

	std::array<BankRAMDevice*, 2> bankram /*{}*/;
};
